/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver
 * File:    Server.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver;

import java.io.FileInputStream;
import java.io.IOException;
import java.net.Socket;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.SecureRandom;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;

import javax.net.ServerSocketFactory;
import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.security.cert.X509Certificate;
import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;

import de.fhhannover.inform.iron.mapserver.communication.BasicClient;
import de.fhhannover.inform.iron.mapserver.communication.Client;
import de.fhhannover.inform.iron.mapserver.communication.CredentialClient;
import de.fhhannover.inform.iron.mapserver.communication.Usermanager;
import de.fhhannover.inform.iron.mapserver.communication.connections.ConnectionContainer;
import de.fhhannover.inform.iron.mapserver.communication.connections.SSRC;
import de.fhhannover.inform.iron.mapserver.configuration.ConfigurationReader;
import de.fhhannover.inform.iron.mapserver.logging.Logging;

public class Server {

		private ConfigurationReader mConf = ConfigurationReader.getInstance();
		private SSLServerSocket mBasicAuth;
		private SSLServerSocket mCredentialAuth;
		private Usermanager mUsermanager = Usermanager.getInstance();
		private static Logger logger;
		static {
			logger = Logging.getTheLogger();
		}
		
		public Server() throws ServerInitialException, NoSuchAlgorithmException, 
			NoSuchProviderException, IOException, KeyManagementException, 
			KeyStoreException, CertificateException, UnrecoverableKeyException {

			logger.info(mConf.getProperty("keystorePath"));
			System.setProperty("javax.net.ssl.keyStore", mConf.getProperty("keystorePath"));
			System.setProperty("javax.net.ssl.keyStorePassword",mConf.getProperty("keystorePW"));
			
			ServerSocketFactory ssocketFactory = SSLServerSocketFactory.getDefault();
			
			if (mConf.containsKey("BasicAuthPort")){
				int basicAuthPort = Integer.parseInt(mConf.getProperty("BasicAuthPort"));
				try {
					mBasicAuth = (SSLServerSocket)ssocketFactory.createServerSocket(basicAuthPort);
				} catch(Exception e) {
					e.printStackTrace();
					throw new ServerInitialException("Internal Server error");
				}				
			}
			
			if(mConf.containsKey("CredentialAuthPort")) {
				System.setProperty("javax.net.ssl.trustStore", mConf.getProperty("truststorePath"));
				System.setProperty("javax.net.ssl.trustStorePassword",mConf.getProperty("truststorePW"));
				
//			    String keyManageProvider="JSSE";

			    SecureRandom random = SecureRandom.getInstance("SHA1PRNG", "SUN");

//			    String provider="JSSE";
			    String proto="TLS";
			    SSLContext con = SSLContext.getInstance(proto);
			    
			    FileInputStream fisKeystore = new FileInputStream( mConf.getProperty("keystorePath") );
			    
			    // make an  instance of the keystore 
			    logger.info(KeyStore.getDefaultType());

			    KeyStore ks = KeyStore.getInstance(KeyStore.getDefaultType());

			    // Now load the keystore. I have yet to find a parameter other
			    // thatn null that works in the second parameter
			    ks.load(fisKeystore, null);

			    // Key manager algorithm this is currently the only default algorithn
			    // that works with JSSE
			    String keyManageAlgo="SunX509";

			    // Get the key manager factory as we have the preliminaries out of the way.
			    KeyManagerFactory km =KeyManagerFactory.getInstance(keyManageAlgo);
			    // intialize the key manager factory
			    km.init(ks , mConf.getProperty("keystorePW").toCharArray());

			    // Now get the key managers
			    KeyManager[] keymanage = km.getKeyManagers();
			    for (int i=0; i<keymanage.length; i++)
			    	System.out.println("keymanage " +keymanage[i]);
			    
			    FileInputStream fisTruststore = new FileInputStream( mConf.getProperty("truststorePath"));
			    logger.info(KeyStore.getDefaultType());
			    KeyStore ts = KeyStore.getInstance(KeyStore.getDefaultType());
			    ts.load(fisTruststore, null);
			    
			    // Now get the Trust Manager stuff
			    TrustManagerFactory tmFactory = TrustManagerFactory.getInstance(keyManageAlgo);
			    tmFactory.init(ts);
			    TrustManager[] tmArray = tmFactory.getTrustManagers();
			    for (int i=0; i<tmArray.length; i++)
			    	System.out.println("tmArray " +tmArray[i]);
			    
			    // Now intialize the keymanagers
			    con.init(keymanage, tmArray, random);
			    
			    // finally we can create a socket factory
			    SSLServerSocketFactory sf = con.getServerSocketFactory();
			    int credentialAuthPort = Integer.parseInt(mConf.getProperty("CredentialAuthPort"));
//			    int basicAuthPort = Integer.parseInt(mConf.getProperty("BasicAuthPort"));
			   // ServerSocket                s = sf.createServerSocket ( credentialAuthPort);

				try{
					mCredentialAuth = (SSLServerSocket)sf.createServerSocket(credentialAuthPort);
					mCredentialAuth.setWantClientAuth(true);
					mCredentialAuth.setNeedClientAuth(true);
					
					
					
				} catch(Exception e) {
					e.printStackTrace();
					throw new ServerInitialException("Internal Server error");
				}
			}
			
			if(mBasicAuth == null && mCredentialAuth == null) {
				throw new ServerInitialException("No Ports in property file specified. Expected basic and/or" +
						" credential authentication");
			}
		}
		
		public static void main(String[] args) throws NoSuchAlgorithmException, IOException, JAXBException {
			
			try {
				Server server = new Server();
				server.start();
			} catch (ServerInitialException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (KeyManagementException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (UnrecoverableKeyException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (NoSuchProviderException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (KeyStoreException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (CertificateException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		public void start() {
			
			if (mBasicAuth != null){
				Thread basic = new Thread(new Runnable() {
					
					@Override
					public void run() {
						while(true){
							Socket basicSocket;
							try {
								basicSocket = mBasicAuth.accept();
								logger.info("New basic connection from "
										+ basicSocket.getInetAddress().getHostAddress()
										+ ":"
										+ basicSocket.getPort());
								Client client;
								try {
									client = new BasicClient(basicSocket);
									new Thread(client).start();
								} catch (JAXBException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
							} catch (IOException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
							
						}
					}
				});
				basic.start();
				logger.info("Started BasicServer on Port " + mBasicAuth.getLocalPort());
			}
			
			if (mCredentialAuth != null){
				Thread credential = new Thread(new Runnable(){

					@Override
					public void run() {
						while(true){
							Socket credentialSocket;
							try {
								credentialSocket = mCredentialAuth.accept();
								logger.info("New credential connection from " + credentialSocket.getInetAddress());
								X509Certificate[] x509 =  ((SSLSocket)credentialSocket).getSession().getPeerCertificateChain();
								logger.info(x509[0].getIssuerDN());
								Client client = null;

								client = new CredentialClient(credentialSocket);

								// Checking if SessionTimer is running
								// TODO Make it nice
								try {
									SSRC s =ConnectionContainer.getInstance().getSessionByPublisherID(x509[0].getIssuerDN().getName());
									s.stopTimer();
								}catch(Exception e) {
									//nothing to do
								}
								//--------------------------------
								
								
								mUsermanager.addClient(x509[0].getIssuerDN().getName(), client);
								new Thread(client).start();

							} catch (IOException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
								logger.debug("HIER FEHLERMELDUNG");
							} catch (JAXBException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
								// Mit dem Server stimmt was nicht
							}
						}
					}
				});
				credential.start();
				logger.info("Started CredentialServer on Port " + mCredentialAuth.getLocalPort());
			}
		}
}