/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.communication
 * File:    CredentialClient.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.communication;

import java.net.Socket;

import javax.net.ssl.SSLPeerUnverifiedException;
import javax.net.ssl.SSLSocket;
import javax.security.cert.X509Certificate;
import javax.xml.bind.JAXBException;

public class CredentialClient extends Client {
	
	public CredentialClient(Socket socket) throws JAXBException {
		super(socket);
	}

	@Override
	protected boolean authorize(String value) {
		return true;
	}

	@Override
	protected boolean isAuthorized() {
		return true;
	}

	@Override
	protected String determinePublisherId() {
		X509Certificate[] x509;
		try {
			x509 = ((SSLSocket)getSocket()).getSession().getPeerCertificateChain();
		} catch (SSLPeerUnverifiedException e) {
			// TODO Auto-generated catch block
			return null;
		}
		String dn = x509[0].getIssuerDN().toString();
		logger.info("Accepted credential client with publisherID " + dn);
		return dn;
	}
}