/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.communication
 * File:    MessageFactory.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.communication;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import org.trustedcomputinggroup._2010.ifmap._2.NewSessionRequestType;
import org.w3._2003._05.soap_envelope.Body;
import org.w3._2003._05.soap_envelope.Envelope;
import org.w3._2003._05.soap_envelope.Header;
import org.w3._2003._05.soap_envelope.ObjectFactory;


public class MessageFactory {
	private JAXBContext mJc; 
	private Marshaller mMarshaller;
	private Unmarshaller mUnmarshaller;
	private int lastResponseSize;
	public MessageFactory() throws JAXBException{
		mJc = JAXBContext.newInstance(Envelope.class,
				Header.class,
				Body.class,
				NewSessionRequestType.class
				);
		mMarshaller = mJc.createMarshaller();
		mUnmarshaller = mJc.createUnmarshaller();
	}
	
	public Envelope getRequest(InputStream is) throws JAXBException {
		Envelope env = (Envelope)((JAXBElement<Envelope>)(mUnmarshaller.unmarshal(is))).getValue();
		return env;
	}
	
	public InputStream getResponse(Envelope env) throws JAXBException {
		ObjectFactory of = new ObjectFactory();
		// don't ask :-(
		JAXBElement<Envelope> jaxEnv = of.createEnvelope(env);
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		mMarshaller.marshal(jaxEnv, baos);
		lastResponseSize = baos.size();
		ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
		return bais;
	}
	/*
	public void sendUnauthorized(PrintWriter pw) {
		HTTPHandler.sendUnauthorized(pw);
	}
*/
	public int getResponseSize() {
		return lastResponseSize;
	}
}