/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.communication.connections
 * File:    ConnectionContainer.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.communication.connections;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Date;
import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;

import org.apache.log4j.Logger;

import de.fhhannover.inform.iron.mapserver.communication.services.ssrc.exceptions.SessionNotFoundException;
import de.fhhannover.inform.iron.mapserver.configuration.communication.ConfigurationManager;
import de.fhhannover.inform.iron.mapserver.logging.Logging;

/**
 * ConnectionContainer class
 * 
 * Connection Handler
 * Stores active connections and handle publisherIDs
 * 
 * @author Waldemar Bender
 * @version 0.1
 * 
 * created: 08.12.2009
 *
 * changes:
 *
 */

public class ConnectionContainer {
	
	private static ConnectionContainer mInstance;
	private ArrayList<String> mSessionIDs = new ArrayList<String>();
	private ArrayList<SSRC> mSSRCs = new ArrayList<SSRC>();
	
	private static ConfigurationManager mConfManager;

	private Random mRandom = new Random();
	private long mSeed = new Date().getTime() -2000;
	private MessageDigest mDigest;
	private static Logger logger;
	
	private Timer connPrintTimer;
	static {
		logger = Logging.getTheLogger();
	}
	
	private ConnectionContainer() throws NoSuchAlgorithmException{
	
		mDigest = java.security.MessageDigest.getInstance("MD5");
		connPrintTimer = new Timer();
		connPrintTimer.scheduleAtFixedRate(new ConnectionPrinter(), 0, 15000);

	}
	
	public static ConnectionContainer getInstance() throws NoSuchAlgorithmException{
		if (mInstance == null){
			mInstance = new ConnectionContainer();
			
		}
		if (mConfManager == null){
			mConfManager = ConfigurationManager.getInstance();
		}
		
		return mInstance;
	}
	
	

	public SSRC addSSRC(String publisher) throws NoSuchAlgorithmException{
		
    	String publisherID;
    	//fixthis
		//publisherID = ConfigurationManager.getInstance().getPublisherID(getPublishHash(context));
    	publisherID = publisher;
    	if (existSession(publisherID)){
    		try {
				logger.info("Existing connection for client with Publisher ID: " + publisherID +" had a connection. Closing the connection and reopening a new session");
    			SSRC oldSSRC = getSessionByPublisherID(publisherID);
    			oldSSRC.closeConnection();

			} catch (SessionNotFoundException e) {

			}
    	}
    	
    	SSRC newConnection = new SSRC(publisherID, getNewSessionID());
    	synchronized (mSSRCs) {
    		mSSRCs.add(newConnection);    		
    	}
    	printConnection(newConnection);

    	return newConnection;
	}

	public void renewSession(String sessionID) throws SessionNotFoundException{
		getSessionBySessionID(sessionID).renewSession();
	}
	
	public String getNewSessionID (){
		long random;
		byte []randomHash;
		String sesID;
		int j = 0;
		do{
		j++;	
			mSeed += 3416;
			mRandom.setSeed(mSeed);
			mSeed -= 2311;
			random = mRandom.nextLong();
			mDigest.update(toByteArray(random));
			randomHash = mDigest.digest();
	    	StringBuffer strbuf = new StringBuffer();
	    	for (int i = 0; i < randomHash.length; i++) {
	  			strbuf.append(toHexString(randomHash[i]));
	    	}
	    	sesID = strbuf.toString();
		}while(mSessionIDs.contains(sesID));
		
		mSessionIDs.add(sesID);
		return sesID;
		
		
		
	}
	
	
	public boolean hasPublisherID(String publisherID) throws NoSuchAlgorithmException{
		return ConfigurationManager.getInstance().isPublisherID(getPublishHash(), publisherID);
	}
	
	private String getPublishHash() throws NoSuchAlgorithmException{

    
		Object o = null;
    	if(o != null)
    	{
    		X509Certificate clientcert = null;
    		X509Certificate[] certificates = null;
    		
    		certificates = (X509Certificate[]) o ;
    		clientcert = certificates[0];
    		
    		//get MD5 of DN
    		return toMD5(clientcert.getIssuerX500Principal().getName());
    		
    	}
    	else
    	{
    		
    		//String base64auth = request.getHeader("authorization");
    		//base64auth = base64auth.split(" ")[1];
    		return "FIXME PublisherHash FIXME";
    	}
		
	}
	
	public SSRC getSessionBySessionID (String sessionID) throws SessionNotFoundException{
		synchronized (mSSRCs) {
			for (int i = 0; i < mSSRCs.size(); i++) {
				if (mSSRCs.get(i).getSessionID().equals(sessionID)){
					return mSSRCs.get(i);
				}
			}
		}
		throw new SessionNotFoundException("No Session with SessionID: " + sessionID);
	}
	
	public SSRC getSessionByPublisherID (String publisherID) throws SessionNotFoundException{
		synchronized (mSSRCs) {
			for (int i = 0; i < mSSRCs.size(); i++) {
				if (mSSRCs.get(i).getPublisherID().equals(publisherID)){
					return mSSRCs.get(i);
				}
			}
		}
		throw new SessionNotFoundException("No Publisher with PublisherID: " + publisherID);
	}
	
	public boolean existSession(String pubID){
		synchronized (mSSRCs) {
			for (int i = 0; i < mSSRCs.size(); i++) {
				if (mSSRCs.get(i).getPublisherID().equals(pubID)){
					return true;
				}
			}
		}
		return false;
	}
	
	public void closeSession(SSRC session){
		synchronized (mSSRCs) {
			mSSRCs.remove(session);
		}
	}
	
	
	private String toMD5 (String hashString) throws  NoSuchAlgorithmException{
		byte [] hashByte = hashString.getBytes();
		MessageDigest digest = java.security.MessageDigest.getInstance("MD5");
		digest.update(hashByte);
		byte[] hash = digest.digest();

		StringBuffer strbuf = new StringBuffer();
		for (int i = 0; i < hash.length; i++) {
			strbuf.append(toHexString(hash[i]));
		}
		return strbuf.toString();
		
	}
	
  	private String toHexString(byte b) {
   		int value = (b & 0x7F) + (b < 0 ? 128 : 0);
	    
    		String ret = (value < 16 ? "0" : "");
    		ret += Integer.toHexString(value).toUpperCase();
	    
		return ret;
  	}
  	
  	private byte[] toByteArray(long l) {
  		byte[] ba=new byte[8];
  		for(int i=0; i<8; i++) {
  			ba[i] = new Long((l>>(i<<3))&255L).byteValue();
  		}
  		return ba;
  	}
  	
  	private void printConnection(SSRC newConnection) {
    	logger.info("NewSession: PublisherID: " + newConnection.getPublisherID() + " SessionID: " + newConnection.getSessionID());
	}
  	

  	 private class ConnectionPrinter extends TimerTask {
  		 
		@Override
		public void run() {
			synchronized (mSSRCs) {
				logger.info("Currently active IF-MAP Sessions:");
				for (SSRC ssrc : mSSRCs) {
					logger.info(ssrc.toString());
				}
			}
		}
  		 
  	 }
  	 

}
