/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.communication.services.ssrc
 * File:    NewSessionReceive.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.communication.services.ssrc;

import java.security.NoSuchAlgorithmException;

import org.apache.log4j.Logger;
import org.trustedcomputinggroup._2010.ifmap._2.NewSessionRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.NewSessionResultType;
import org.trustedcomputinggroup._2010.ifmap._2.ResponseType;

import de.fhhannover.inform.iron.mapserver.communication.DeprecatedDataModelServiceAccessor;
import de.fhhannover.inform.iron.mapserver.communication.connections.ConnectionContainer;
import de.fhhannover.inform.iron.mapserver.communication.connections.SSRC;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.ParameterException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.PublisherConstructionException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.RequestCreationException;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.RunningSessionException;
import de.fhhannover.inform.iron.mapserver.logging.Logging;
import de.fhhannover.inform.iron.mapserver.messages.NewSessionRequest;
import de.fhhannover.inform.iron.mapserver.messages.RequestFactory;

/**
 * NewSessionReceive class
 * 
 * Add MAPC to ConnectionContainer
 * 
 * @author dl
 * @version 0.1
 * 
 * created: ??.??.2009
 *
 * changes:
 *  04.03.2010 - error responses
 *  17.02.2010 - cleanup, added logger
 *
 */

public class NewSessionReceive {
	private static Logger logger;
	static {
		logger = Logging.getTheLogger();
	}
	
	public static ResponseType newSession(NewSessionRequestType newSession, String publisher) 
	throws NoSuchAlgorithmException, RequestCreationException, 
	ParameterException, RunningSessionException, PublisherConstructionException {

		
		RequestFactory requestFactory = RequestFactory.getInstance();
		
    	ResponseType newSessionResponseType = new ResponseType();
    	NewSessionResultType newSessionResultType =  new NewSessionResultType();

    	if(newSession.getMaxPollResultSize() != null)
    	{
    		//TODO: Richtige MaxPollResultSize zurueckgeben
    		//set maxPollResultType
    		newSessionResultType.setMaxPollResultType(newSession.getMaxPollResultSize());
    	}

    	SSRC newConnection = null;
    	newConnection =  ConnectionContainer.getInstance().addSSRC(publisher);
    	
    	//set publisherID and sessionID
    	newSessionResultType.setIfmapPublisherId(newConnection.getPublisherID());
    	newSessionResultType.setSessionId(newConnection.getSessionID());
    	newSessionResponseType.setNewSessionResult(newSessionResultType);

    	NewSessionRequest sessionReq = requestFactory.createNewSessionRequest(
    			newSessionResultType.getSessionId(), newSessionResultType.getIfmapPublisherId(),
    			newSessionResultType.getMaxPollResultType());
    	
    	DeprecatedDataModelServiceAccessor.getDataModelServiceInstance().newSession(sessionReq);
    	
    	return newSessionResponseType;
	}
}
