/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.datamodel
 * File:    IdentifierRep.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.datamodel;

import java.util.Collection;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.Identifier;
import de.fhhannover.inform.iron.mapserver.utils.LinkedMap;

/**
 * IdentifierRep
 *
 * @version 0.1
 * @author awe, vp
 */

/*
 * created: 19.11.09
 * changes:
 * 	22.11.09 vp - Added: identifierrepository, getInstance()
 *  26.11.09 vp - empty methods implemented
 *  10.12.09 vp - addIdentifier() DEEP COPY of identifier
 *  12.12.09 vp - addIdentifier() changed to private and return identifier, getOrCreateIdentifier() changed#
 *  22.12.09 vp - removeIdentifier bug fixed 
 *  03.01.10 vp - new datastructure
 *  11.02.10 vp - the visibility of the classes and methods is changed
 *  30.04.10 aw - remove singletons!
 */
class IdentifierRep {

	private LinkedMap<Identifier> identifiers;
	private long timestamp;
	
	IdentifierRep() {
		identifiers = new LinkedMap<Identifier>();
		timestamp = System.currentTimeMillis();
	}
	
	/**
	 * Add new identifier to identifier repository
	 * DEEP COPY
	 * @param ident
	 */
	private Identifier addIdentifier(Identifier ident) {
		Identifier toadd = ident.copy();
		identifiers.put(toadd.hashCode(), toadd);
		return toadd;
	}

	/**
	 * If ident exist return it, if ident does not existed 
	 * create new ident and return it
	 * @param ident
	 * @return
	 */
	Identifier getOrCreateIdentifier(Identifier ident) {
		if (ident == null) {
			return null;
		}
		
		Identifier tempIdent = getIdentifier(ident);
		if (tempIdent == null) {
			tempIdent = addIdentifier(ident);
		}
		return tempIdent;
	}
	
	/**
	 * Get identifier from identifier repository
	 * @param ident
	 * @return identifier from identifier repository
	 */
	Identifier getIdentifier(Identifier ident) {
		if (ident == null) {
			return null;
		}
		return identifiers.getElement(ident.hashCode(), ident);
	}
	
	/**
	 * Returns all linked Identifier
	 * @return an array of identifiers
	 */
	Identifier[] getAllIdentifiers(){
		Identifier[] idents = new Identifier[identifiers.size()];
		Collection<Identifier> list = identifiers.values();
		int i=0;
		for (Identifier id : list) {
			idents[i++] = id;
		}
		return idents;
	}
	
	public long getTimestamp() {
		return timestamp;
	}

	public void setTimestamp(long timestamp) {
		this.timestamp = timestamp;
	}

	/**
	 * Remove identifier from identifier repository
	 * @param ident
	 */
	void removeIdentifier(Identifier ident) {
		if (ident != null) {
			identifiers.remove(ident.hashCode(), ident);
		}
	}
	
	String graphvizNodes() {
		StringBuffer sb = new StringBuffer();
		sb.append("node [shape=ellipse] \n");
		
		Collection<Identifier> ilist = identifiers.values();
		
		for (Identifier i : ilist) {
			sb.append("\t{node [label=\"");
			sb.append(i.toString());
			sb.append("\"] ");
			sb.append((long)((long)i.hashCode() * (long)i.hashCode()));
			sb.append("; }\n");
		}
		return sb.toString();
	}
}
 
 