/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.datamodel.graph
 * File:    IdentifierType.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.datamodel.graph;

import java.util.HashMap;

import de.fhhannover.inform.iron.mapserver.datamodel.graph.meta.MetaType;

/**
 * A class that represents the different identifier types
 * with a string. This is later used in search to test if
 * a identifier is contained in the terminal-identifier-type
 * string.
 * 
 * 
 * @author aw
 * @version 0.1
 * 
 * created: 17.12.09
 * changes:
 * 	17.12.09 aw - created this class with instanceOfTypeWith()
 *		Mostly copied from MetaDataType
 *  04.02.10 aw - added some comments
 *
 */
public class IdentifierType {
 
	private String typestring;
	
	private IdentifierType(String types) {
		typestring = new String(types);
	}

	/**
	 * Storage for the different strings. Won't be large as the
	 * number of identifiers is not that high.
	 */
	private static HashMap<Integer, IdentifierType> types =
		new HashMap<Integer, IdentifierType>(10);
	
	/**
	 * Get the type for a given typestring.
	 * If a type with the same typestring exists this IdentifierType object
	 * will be returned, else a new one will be constructed and stored
	 * and thereafter returned.
	 * Returns null if the given typestring was null
	 * 
	 * @param typestring
	 * @return
	 */
	public static IdentifierType instanceOfTypeWith(String typestring) {
		IdentifierType t;
		if (typestring != null && typestring.length() != 0) {
			t = types.get(typestring.hashCode());
			if (t == null) {
				t = new IdentifierType(typestring);
				types.put(typestring.hashCode(), t);
			}
			return t;
		}
		return null;
	}
	
	public String getTypeString() {
		return new String(typestring);
	}
	
	public boolean equals(Object o) {
		MetaType ot;
		if (o == null)
			return false;
		
		if (this == o)
			return true;
		
		if (o instanceof MetaType) {
			ot = (MetaType) o;
			if ((ot.getTypeString().equals(this.getTypeString()))) {
				return true;
			}
			return false;
		} else {
			return false;
		}
	}
}
