/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.datamodel.graph.meta
 * File:    MetaType.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.datamodel.graph.meta;

import java.util.HashMap;

import org.apache.log4j.Logger;

import de.fhhannover.inform.iron.mapserver.logging.Logging;

/**
 * A simple class that represents different MetaDataTypes.
 * Use it when comparing if Metadata is of the same type.
 * 
 * Metadata should have a typestring of urn:type
 * where urn might be http://example.com and type the Metadatatype
 * 
 * The standardmetadatatypes use:
 * urn:trustedcomputinggroup.org:2010:IFMAP-METADATA:2 as urn
 * and role, event, mac-ip ... as types.
 * 
 * @author aw
 * @version 0.1
 */
/*
 * 
 * created: 05.12.09
 * changes:
 * 	05.12.09 aw - First version of this class.
 *  17.12.09 aw - Fixed typestring not set
 *  10.04.10 aw - Added some logging...
 *  13.06.10 aw - Visibility change
 *
 */
public class MetaType {
	
	/**
	 * a static logger
	 */
	private static Logger logger = Logging.getTheLogger();
	
	/**
	 * hashmap for all typestrings
	 */
	private static HashMap<Integer, MetaType> types =
											new HashMap<Integer, MetaType>();

	private String typestring;
	
	private MetaType(String typestring) {
		this.typestring = new String(typestring);
		
	}
	
	
	/**
	 * Get the type for a given typestring.
	 * If a type with the same typestring exists this will be returned,
	 * else a new one will be constructed and stored.
	 * 
	 * @param typestring
	 * @return
	 */
	static MetaType instanceOfTypeWith(String typestring) {
		MetaType t = null;
		if (typestring != null) {
			t = types.get(typestring.hashCode());
			if (t == null) {
				logger.trace("New MetaDataType with typestring=" + typestring);
				t = new MetaType(typestring);
				types.put(typestring.hashCode(), t);
			}
		}
		return t;
	}

	
	public String getTypeString() {
		return new String(typestring);
	}

	
	public boolean equals(Object o) {
		MetaType ot;
		if (o == null)
			return false;
		
		if (this == o)
			return true;
		
		if (o instanceof MetaType) {
			ot = (MetaType) o;
			return ot.getTypeString().equals(this.getTypeString());
		} else {
			return false;
		}
	}
}
