/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.datamodel.graph.meta
 * File:    XMLMetadata.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

 package de.fhhannover.inform.iron.mapserver.datamodel.graph.meta;

import java.util.List;

import org.jaxen.JaxenException;
import org.jaxen.SimpleNamespaceContext;
import org.jaxen.jdom.JDOMXPath;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.output.DOMOutputter;

import de.fhhannover.inform.iron.mapserver.datamodel.Filter;
import de.fhhannover.inform.iron.mapserver.datamodel.exceptions.MetadataConstructionException;

/**
 * Concrete Metadata Class Implementation
 * 
 * Concrete Metadata class which *hopefully*
 * uses JDOM objects and JDOMs builtin XPath support
 * to get matchFilter() to work.
 * 
 *
 * FIXME:
 * Warning, this class is dangerous. Try to initialize
 * everything with the correct parameters.
 * 
 * @author aw
 * @version 0.1
 */

/*
 * created: 28.11.09
 * changes:
 * 	28.11.09 aw - First version of this class.
 *  05.12.09 aw - Added type to the metadata, setting of cardinality forced
 * 		while constructing.
 *	09.12.09 aw - Made copy constructor private
 *  04.02.10 aw - Formatting
 *  10.02.10 aw - some cleanup
 *  11.02.10 aw,dl - added toXMLString()
 *  18.02.10 aw - added namespace support, removed toXMLString()
 *  10.04.10 aw - implement toW3CDocument(),
 *  			construct with known lifetime
 *  11.04.10 aw - modify to use new constructor, setTimestamp() at the end of
 *                the constructor.
 *
 */
public class XMLMetadata extends Metadata {
	
	private Document xmlroot;
	
	/**
	 * Construct a XMLMetadata object from a jdom document.
	 * Be aware that doc needs to have at least one element else
	 * jdom will throw a exception.
	 * 
	 * @throws MetadataConstructionException 
	 */
	public XMLMetadata(Document doc, MetaLifeTimeType lifetime, String timestamp)
											throws MetadataConstructionException, Exception {
		
		super(MetaType.instanceOfTypeWith(((Element)doc.getContent().get(0))	// FIXME: dangerous
												.getNamespaceURI() + ":" + ((Element)doc.getContent().get(0))
																	.getName()), lifetime, timestamp);
		if (doc == null) {
			throw new MetadataConstructionException("XMLMetadata: " +
					"doc is null");
		}
		if (doc.getContentSize() == 0) {
			throw new MetadataConstructionException("XMLMetadata: " +
					"no content!");
		}
		xmlroot = (Document) doc.clone();
		setTimestamp();
		setGetCardinality();
	}
	

	private XMLMetadata(XMLMetadata xmlMetadata) throws MetadataConstructionException {
		super(xmlMetadata);
		xmlroot = (Document) xmlMetadata.getXmlRoot().clone();
	}


	/**
	 * FIXME: Review this
	 */
	@Override
	public void setPublisherId(String pubid) {
		if (pubid != null) {
			Element el = (Element) xmlroot.getContent(0);
			el.setAttribute("ifmap-publisher-id", pubid);
		}
	}


	/**
	 * FIXME: Review this
	 * @throws MetadataConstructionException 
	 */
	@Override
	public void setTimestamp() throws MetadataConstructionException {
		if (getTimestamp() != null && getTimestamp().length() > 0) {
			//logger.trace("setTimestamp() with string=" + getTimestamp());
			Element el = (Element) xmlroot.getContent(0);
			el.setAttribute("ifmap-timestamp", getTimestamp());
			//logger.trace("attribute ifmap-timestamp=" + el.getAttributeValue("ifmap-timestamp"));
		} else {
			throw new MetadataConstructionException("No timestamp given");
		}
	}

	/**
	 * FIXME: Ugly
	 */
	@SuppressWarnings("unchecked")
	@Override
	public boolean matchesFilter(Filter f) {
		JDOMXPath xpath;
		String fs = f.getFilterString();
		List result;
		
		if (f.isMatchEverything()) {
			return true;
		}
		
		if (f.isMatchNothing()) {
			return false;
		}
		
		try {
			xpath = new JDOMXPath(fs);
		} catch (JaxenException e) {
			// something gone wrong!
			return false;
		}
		
		xpath.setNamespaceContext(new SimpleNamespaceContext(f.getNamespaceMap()));
		
		try {
			result = xpath.selectNodes(xmlroot);
		} catch (JaxenException e) {
				// something gone wrong
				return false;
		}
//		logger.trace("result=" + result);
//		if (result != null)
//			logger.trace("result size=" + result.size());
				
	
		// Hope this works...
		if (result != null && result.size() > 0) {
			if (result.size() == 1 && (result.get(0) instanceof Boolean)) {
//				logger.trace("result list is Boolean list... hopefully...");
				if ((Boolean)result.get(0)) {
//					logger.trace("element zero true");
					return true;
				} else {
					return false;
				}
			} else {
//				logger.trace("result list is node list... hopefully...");
				return true;
			}
		}
		return false;
	}
	
	
	public Document getXmlRoot() {
		return xmlroot;
	}


	@Override
	public Metadata copy() {
		try {
			return new XMLMetadata(this);
		} catch (MetadataConstructionException e) {
			e.printStackTrace();
		}
		return null;
	}


	public void setGetCardinality() throws MetadataConstructionException {
		String message = "no element";
		Element el = (Element) xmlroot.getContent().get(0);
		if (el != null) {
			String cardString = el.getAttributeValue(CARDINALITYSTRING);
			if (cardString != null) {
				if (cardString.equals(CARDINALITYMULTI)) {
					cardinality = MetaCardinalityType.multi;
					return;
				} else if (cardString.equals(CARDINALITYSINGLE)) {
					cardinality = MetaCardinalityType.single;
					return;
				}
			}
			message = "attribute ifmap-cardinality not found";
		}
		throw new MetadataConstructionException(message);
	}


	@Override
	public org.w3c.dom.Document toW3CDocument() {
		org.w3c.dom.Document ret = null;
		DOMOutputter outputter = new DOMOutputter();
		
		try {
			ret = outputter.output(getXmlRoot());
		} catch (JDOMException e) {
			logger.fatal("Could not convert Metadata to W3C Document");
		}
		return ret;
	}
}
